#import "LgcrePlugin.h"
#import <LGCRE/LGCRE.h>

@implementation LgcrePlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"lgcre"
            binaryMessenger:[registrar messenger]];
  LgcrePlugin* instance = [[LgcrePlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } 
  else if ([@"init" isEqualToString:call.method]) {
      NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
      
    NSString *libraryPath = ([paths count] > 0) ? [paths objectAtIndex:0] : nil;
    NSString *workspace = [libraryPath stringByAppendingPathComponent:@"Lgcre"];
    NSFileManager *fileManager = [NSFileManager defaultManager];
      [fileManager createDirectoryAtPath: workspace
                    withIntermediateDirectories:TRUE
                              attributes:nil
                                   error:nil];
	NSMutableDictionary* options = [call arguments];
	[options setObject: workspace forKey: @"lgcre.workspace"];
	LGCREInitWithOptions(options);
  } 
  else if ([@"reset" isEqualToString:call.method]) {
	LGCREReset();
  } 
  else if ([@"release" isEqualToString:call.method]) {
	LGCRERelease();
  } 
  else if ([@"run" isEqualToString:call.method]) {
      LGCREQuery* query = [[LGCREQuery alloc] init];
            NSDictionary* arguments = [call arguments];
            for(NSString* key in arguments)
            {
                if(![key isEqualToString:@"pois"])
              {
                  result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ is unknown", key]
                                        message:nil
                                        details:nil]);
                  return;
            
              }
            }

            NSArray* pois = (NSArray*)arguments[@"pois"];
            for(NSDictionary* entry in pois)
            {
              for(NSString* key in entry)
              {
                if(![key isEqualToString:@"poi"] &&
                   ![key isEqualToString:@"categories"])
                {
                    result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ is unknown", key]
                                          message:nil
                                          details:nil]);
                    return;
                }
              }

              NSString* poi = (NSString*)entry[@"poi"];
              NSArray* categories = (NSArray*)entry[@"categories"];
              for(NSString* category in categories)
              {
                  [query setPoiCategory: poi cat: category];
              }
            }

            NSMutableArray* ret = [[NSMutableArray alloc] init];
            NSArray* entries = LGCRERun(query);
            query= nil;
            if(entries != nil)
            {
              int count= 0;
              for(LGCREQueryResultEntry* entry in entries)
              {
                  NSString* str = [NSString stringWithFormat:@"{\"poi\" : \"%@\", \"score\" : %f} ",
                                   [entry getPoi],
                                   [entry getCategory]
                                   ];
                
                  [ret addObject: str];
                count++;
              }
              result(ret);
            }
            else
            {
              result(nil);
            }

  } 
  else if ([@"addData" isEqualToString:call.method]) {
      NSDictionary* arguments = [call arguments];
      
      for (NSString *key in arguments)
      {
	      if(![key isEqualToString:@"pois"] &&
	  	     ![key isEqualToString:@"categories"] &&
		     ![key isEqualToString:@"links"] &&
		     ![key isEqualToString:@"links_categories"] &&
		     ![key isEqualToString:@"links_poi_category"] &&
		     ![key isEqualToString:@"person"])
	      {
              result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ is unknown", key]
                                    message:nil
                                    details:nil]);
		      return;
	      }
      }

      NSArray* pois = (NSArray*)arguments[@"pois"];
      if(pois != nil)
      {
	      for (NSString* poi in pois)
	      {
		      LGCREAddPoi(poi);
	      }
      }

      NSArray* categories = (NSArray*)arguments[@"categories"];
      if(categories != nil)
      {
	      for (NSString* category in categories)
	      {
		      LGCREAddCategory(category);
	      }
      }
      
      NSDictionary* link_categories = (NSDictionary*)arguments[@"links_categories"];
      if(link_categories != nil)
      {
	      for(NSDictionary* link in link_categories)
	      {
		      for(NSString* key in link)
		      {
                  if(![key isEqualToString:@"category1"] &&
                     ![key isEqualToString:@"category2"])
			      {
                      result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in links_categories is unknown", key]
                                            message:nil
                                            details:nil]);
                      return;
			      }
		      }
		      LGCREAddLinkCategoryCategory(link[@"category1"], link[@"category2"]);
	      }
      }

      NSDictionary* link_poi_categories = (NSDictionary*) arguments[@"links_poi_category"];
      if(link_poi_categories != nil)
      {
	      for(NSDictionary* link in link_poi_categories)
	      {
		      for(NSString* key in link)
		      {
                  if(![key isEqualToString:@"poi"] &&
                     ![key isEqualToString:@"category"])
			      {
                      result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in links_poi_categories is unknown", key]
                                            message:nil
                                            details:nil]);
                      return;
			      }
		      }
		      LGCREAddLinkPoiCategory(link[@"poi"], link[@"category"]);
	      }
      }
      
      NSDictionary* person_data = (NSDictionary*)arguments[@"person"];
      if(person_data !=  nil)
      {
	      for(NSString* key in person_data)
	      {
              if(![key isEqualToString:@"pois"] &&
                 ![key isEqualToString:@"categories"])
		      {
                  result([FlutterError errorWithCode:[NSString stringWithFormat:@"Data key %@ in person entry is unknown", key]
                                        message:nil
                                        details:nil]);
                  return;
		      }
	      }

	      NSArray* person_pois = (NSArray*)person_data[@"pois"];
	      if(pois != nil)
	      {
		      for (NSString* poi in pois)
		      {
			      LGCREAddLinkPersonPoi(poi);
		      }
	      }

	      NSArray* person_categories = (NSArray*)person_data[@"categories"];
	      if(categories != nil)
	      {
		      for (NSString* category in categories)
		      {
			      LGCREAddLinkPersonCategory(category);
		      }
	      }
      }
  } 
  else {
	  result(FlutterMethodNotImplemented);
  }
}

@end
